from __future__ import annotations
import sys
from importlib.machinery import ModuleSpec, PathFinder
from importlib.machinery import all_suffixes as module_suffixes
from importlib.util import spec_from_file_location
from itertools import chain
from pathlib import Path

MAPPING: dict[str, str] = {'stegopy': 'E:\\Scripts\\stegopy\\stegopy', 'tests': 'E:\\Scripts\\stegopy\\tests', 'venv': 'E:\\Scripts\\stegopy\\venv'}
NAMESPACES: dict[str, list[str]] = {'tests': ['E:\\Scripts\\stegopy\\tests'], 'venv': ['E:\\Scripts\\stegopy\\venv'], 'tests.assets': ['E:\\Scripts\\stegopy\\tests\\assets'], 'venv.Include': ['E:\\Scripts\\stegopy\\venv\\Include'], 'venv.Lib': ['E:\\Scripts\\stegopy\\venv\\Lib'], 'venv.Scripts': ['E:\\Scripts\\stegopy\\venv\\Scripts'], 'venv.Lib.site-packages': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages'], 'venv.Lib.site-packages.jaraco': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\jaraco'], 'venv.Lib.site-packages.venv': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\venv'], 'venv.Lib.site-packages.build.lib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\build\\lib'], 'venv.Lib.site-packages.build.lib.venv': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\build\\lib\\venv'], 'venv.Lib.site-packages.build.lib.venv.Lib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\build\\lib\\venv\\Lib'], 'venv.Lib.site-packages.docutils.parsers.rst.include': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\parsers\\rst\\include'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.big-black': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\big-black'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.big-white': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\big-white'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.default': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\default'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.medium-black': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\medium-black'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.medium-white': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\medium-white'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.small-black': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\small-black'], 'venv.Lib.site-packages.docutils.writers.s5_html.themes.small-white': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\docutils\\writers\\s5_html\\themes\\small-white'], 'venv.Lib.site-packages.id._internal': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\id\\_internal'], 'venv.Lib.site-packages.mypy.typeshed': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed'], 'venv.Lib.site-packages.mypy.xml': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\xml'], 'venv.Lib.site-packages.mypy.typeshed.stdlib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib'], 'venv.Lib.site-packages.mypy.typeshed.stubs': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stubs'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.asyncio': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\asyncio'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.collections': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\collections'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.concurrent': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\concurrent'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.ctypes': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\ctypes'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.curses': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\curses'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.dbm': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\dbm'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.distutils': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\distutils'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.email': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\email'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.encodings': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\encodings'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.ensurepip': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\ensurepip'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.html': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\html'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.http': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\http'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.importlib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\importlib'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.json': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\json'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.lib2to3': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\lib2to3'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.logging': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\logging'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.msilib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\msilib'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.multiprocessing': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\multiprocessing'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.os': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\os'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.pydoc_data': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\pydoc_data'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.pyexpat': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\pyexpat'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.sqlite3': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\sqlite3'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.sys': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\sys'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.tkinter': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\tkinter'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.unittest': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\unittest'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.urllib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\urllib'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.venv': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\venv'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.wsgiref': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\wsgiref'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xml': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xml'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xmlrpc': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xmlrpc'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.zipfile': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\zipfile'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.zoneinfo': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\zoneinfo'], 'venv.Lib.site-packages.mypy.typeshed.stdlib._typeshed': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\_typeshed'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.concurrent.futures': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\concurrent\\futures'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.ctypes.macholib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\ctypes\\macholib'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.distutils.command': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\distutils\\command'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.email.mime': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\email\\mime'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.importlib.metadata': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\importlib\\metadata'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.importlib.resources': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\importlib\\resources'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.lib2to3.fixes': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\lib2to3\\fixes'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.lib2to3.pgen2': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\lib2to3\\pgen2'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.multiprocessing.dummy': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\multiprocessing\\dummy'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xml.dom': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xml\\dom'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xml.etree': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xml\\etree'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xml.parsers': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xml\\parsers'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xml.sax': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xml\\sax'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.xml.parsers.expat': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\xml\\parsers\\expat'], 'venv.Lib.site-packages.mypy.typeshed.stdlib.zipfile._path': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stdlib\\zipfile\\_path'], 'venv.Lib.site-packages.mypy.typeshed.stubs.mypy-extensions': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypy\\typeshed\\stubs\\mypy-extensions'], 'venv.Lib.site-packages.mypyc.lib-rt': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\mypyc\\lib-rt'], 'venv.Lib.site-packages.pkg_resources.tests.data': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\pkg_resources\\tests\\data'], 'venv.Lib.site-packages.pkg_resources.tests.data.my-test-package-source': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\pkg_resources\\tests\\data\\my-test-package-source'], 'venv.Lib.site-packages.pkg_resources.tests.data.my-test-package-zip': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\pkg_resources\\tests\\data\\my-test-package-zip'], 'venv.Lib.site-packages.pkg_resources.tests.data.my-test-package_unpacked-egg': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\pkg_resources\\tests\\data\\my-test-package_unpacked-egg'], 'venv.Lib.site-packages.pkg_resources.tests.data.my-test-package_zipped-egg': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\pkg_resources\\tests\\data\\my-test-package_zipped-egg'], 'venv.Lib.site-packages.setuptools._vendor': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\_vendor'], 'venv.Lib.site-packages.setuptools.tests.indexes': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\tests\\indexes'], 'venv.Lib.site-packages.setuptools.tests.indexes.test_links_priority': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\tests\\indexes\\test_links_priority'], 'venv.Lib.site-packages.setuptools.tests.indexes.test_links_priority.simple': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\tests\\indexes\\test_links_priority\\simple'], 'venv.Lib.site-packages.setuptools.tests.indexes.test_links_priority.simple.foobar': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\tests\\indexes\\test_links_priority\\simple\\foobar'], 'venv.Lib.site-packages.setuptools._distutils.compilers': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\_distutils\\compilers'], 'venv.Lib.site-packages.setuptools._distutils.compilers.C': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\_distutils\\compilers\\C'], 'venv.Lib.site-packages.setuptools._distutils.compilers.C.tests': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\_distutils\\compilers\\C\\tests'], 'venv.Lib.site-packages.setuptools._vendor.jaraco': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\setuptools\\_vendor\\jaraco'], 'venv.Lib.site-packages.venv.Lib': ['E:\\Scripts\\stegopy\\venv\\Lib\\site-packages\\venv\\Lib']}
PATH_PLACEHOLDER = '__editable__.stegopy-0.3.1.finder' + ".__path_hook__"


class _EditableFinder:  # MetaPathFinder
    @classmethod
    def find_spec(cls, fullname: str, path=None, target=None) -> ModuleSpec | None:  # type: ignore
        # Top-level packages and modules (we know these exist in the FS)
        if fullname in MAPPING:
            pkg_path = MAPPING[fullname]
            return cls._find_spec(fullname, Path(pkg_path))

        # Handle immediate children modules (required for namespaces to work)
        # To avoid problems with case sensitivity in the file system we delegate
        # to the importlib.machinery implementation.
        parent, _, child = fullname.rpartition(".")
        if parent and parent in MAPPING:
            return PathFinder.find_spec(fullname, path=[MAPPING[parent]])

        # Other levels of nesting should be handled automatically by importlib
        # using the parent path.
        return None

    @classmethod
    def _find_spec(cls, fullname: str, candidate_path: Path) -> ModuleSpec | None:
        init = candidate_path / "__init__.py"
        candidates = (candidate_path.with_suffix(x) for x in module_suffixes())
        for candidate in chain([init], candidates):
            if candidate.exists():
                return spec_from_file_location(fullname, candidate)
        return None


class _EditableNamespaceFinder:  # PathEntryFinder
    @classmethod
    def _path_hook(cls, path) -> type[_EditableNamespaceFinder]:
        if path == PATH_PLACEHOLDER:
            return cls
        raise ImportError

    @classmethod
    def _paths(cls, fullname: str) -> list[str]:
        paths = NAMESPACES[fullname]
        if not paths and fullname in MAPPING:
            paths = [MAPPING[fullname]]
        # Always add placeholder, for 2 reasons:
        # 1. __path__ cannot be empty for the spec to be considered namespace.
        # 2. In the case of nested namespaces, we need to force
        #    import machinery to query _EditableNamespaceFinder again.
        return [*paths, PATH_PLACEHOLDER]

    @classmethod
    def find_spec(cls, fullname: str, target=None) -> ModuleSpec | None:  # type: ignore
        if fullname in NAMESPACES:
            spec = ModuleSpec(fullname, None, is_package=True)
            spec.submodule_search_locations = cls._paths(fullname)
            return spec
        return None

    @classmethod
    def find_module(cls, _fullname) -> None:
        return None


def install():
    if not any(finder == _EditableFinder for finder in sys.meta_path):
        sys.meta_path.append(_EditableFinder)

    if not NAMESPACES:
        return

    if not any(hook == _EditableNamespaceFinder._path_hook for hook in sys.path_hooks):
        # PathEntryFinder is needed to create NamespaceSpec without private APIS
        sys.path_hooks.append(_EditableNamespaceFinder._path_hook)
    if PATH_PLACEHOLDER not in sys.path:
        sys.path.append(PATH_PLACEHOLDER)  # Used just to trigger the path hook
